// -*- C++ -*- std header                                                      |
//-----------------------------------------------------------------------------+
//                                                                             |
// auxx.h                                                                      |
//                                                                             |
// C++ code                                                                    |
//                                                                             |
// Copyright Walter Dehnen, 2000-2001                                          |
// e-mail:   dehnen@mpia.de                                                    |
// address:  Max-Planck Institut fr Astronomie,                               |
//           Knigstuhl 17, D-69117 Heidelberg, Germany                        |
//                                                                             |
//-----------------------------------------------------------------------------+
//                                                                             |
// defines                                                                     |
//                                                                             |
// some global macros                                                          |
// some global constants                                                       |
// some global typedefs                                                        |
//                                                                             |
//-----------------------------------------------------------------------------+

#ifndef included_auxx_h
#define included_auxx_h

#ifndef included_iostream
#include <iostream>
#define included_iostream
#endif

////////////////////////////////////////////////////////////////////////////////
// 1. INTERNAL PRECISION
//
//
// 1.1 from math.h:
//

extern "C" {
  double sqrt (double);
  double exp  (double);
  double log  (double);
  float  sqrtf(float);
  float  expf (float);
  float  logf (float);
  double pow  (double,double);
//   float  powf (float,float);
}

#ifdef DOUBLEPRECISION
//
// 1.2 double precision: 64bit floating point numbers to hold a real
//TM
namespace fasttree {
  typedef double real;
}

#define SQRT sqrt
#define POW  pow
#define EXP  exp
#define LOG  log

#else // DOUBLEPRECISION
//
// 1.3 single precision: 32bit floating point numbers to hold a real
//
namespace fasttree {
  typedef float real;
}

#ifdef EBUG
#define SQRT(x)   real(sqrt(x))
#define EXP(x)    real(exp(x))
#define LOG(x)    real(log(x))
#else
#define SQRT      sqrtf
#define EXP       expf            // use on linux machines
#define LOG       logf            // use on linux machines
// #define EXP(x)    real(exp(x))       // use on suns
// #define LOG(x)    real(log(x))       // use on suns
#endif
#define POW(x,y)  real(pow(double(x),double(y)))

#endif // DOUBLEPRECISION

////////////////////////////////////////////////////////////////////////////////
//
// 2. DATA TRANSFER VIA BODIES OR ARRAYS & PRECISION USED IN ARRAY ARGS


namespace fasttree {
#ifdef DOUBLEINPUT
  typedef double areal;
#else  // DOUBLEINPUT
  typedef float areal;
#endif // DOUBLEINPUT
}

////////////////////////////////////////////////////////////////////////////////
//
// 3. DIMENSIONALITY
//
// 3.1 Two dimensions (disk)
//
#ifdef TWODIMENSIONAL

#define NDIM 2   // # dimensions
#define NSUB 4   // 2^NDIM

#else // TWODIMENSIONAL
//
// 3.2 Three dimensions
//

#define NDIM 3   // # dimensions
#define NSUB 8   // 2^NDIM

#endif // TWODIMENSIONAL

#ifndef included_vect_h
#include <tree/vect.h>
#endif

#ifndef included_tens_h
#include <tree/tens.h>
#endif

namespace fasttree {
  typedef class vector <real>  vect;
  typedef class evector<real>  evec;
  typedef class sym2<real>     matr;
  typedef class sym3<real>     tens;
  typedef class esym2<real>    emat;
  typedef class esym3<real>    eten;
}

#ifdef TWODIMENSIONAL
namespace fasttree {
  typedef vect                   real;
}
#else // TWODIMENSIONAL
namespace fasttree {
  typedef vect                   amom;
}
#endif // TWODIMENSIONAL



////////////////////////////////////////////////////////////////////////////////
//
// 4. USEFUL CONSTANTS AND TYPEDEFS
//

namespace fasttree {
  typedef unsigned short   indx;         // use only in non-register memory
  typedef unsigned int     uint;
  typedef int              flag_type;
  //----------------------------------------------------------------------------
  inline void error  (const char*m) {
    std::cerr<<" TREE ERROR: "  <<m<<std::endl;
    exit(1);
  }
  //----------------------------------------------------------------------------
  inline void warning(const char*m) {
    std::cerr<<" TREE WARNING: "<<m<<std::endl;
  }
  //----------------------------------------------------------------------------
  const   real zero    = 0., 
               third   = 0.3333333333333333333,
               half    = 0.5,
               one     = 1.,
               two     = 2.,
               ten     = 10.;
}

////////////////////////////////////////////////////////////////////////////////

#endif  // included_auxx_h
