// -*- C++ -*- std header                                                      |
//-----------------------------------------------------------------------------+
//                                                                             |
// open.h                                                                      |
//                                                                             |
// C++ code                                                                    |
//                                                                             |
// Copyright Walter Dehnen, 1999-2001                                          |
// e-mail:   dehnen@mpia.de                                                    |
// address:  Max-Planck Institut fr Astronomie,                               |
//           Knigstuhl 17, D-69117 Heidelberg, Germany                        |
//                                                                             |
//-----------------------------------------------------------------------------+
//                                                                             |
// defines                                                                     |
//                                                                             |
// class opening             see open.cc                                       |
//                                                                             |
//-----------------------------------------------------------------------------+

#ifndef included_open_h
#define included_open_h

#ifndef included_auxx_h
#include <tree/auxx.h>
#endif

////////////////////////////////////////////////////////////////////////////////
namespace fasttree {
  class opening {
  private:
    bool   TH_CONST;
    double TH0, ITH0;
    uint   N;
    double *Z, *lM;
    void set_table(const uint,const real);
    real tabulated_theta(const real) const;
    real tabulated_invtheta(const real) const;
  public:
    opening   (const real, const bool, const uint=1000, const real=1.e-10);
    void reset(const real, const bool, const uint=1000, const real=1.e-10);
    ~opening() { if(Z) delete[] Z; if(lM) delete[] lM; }
    real    theta(const real) const;                          // gives theta(M)
    real invtheta(const real) const;                          // gives 1/theta(M)
    real    theta_min()   const { return TH0; }
    bool    theta_const() const { return TH_CONST; }
    const char*   method()const { return TH_CONST? "theta=const" : "theta(M)"; }
  };

  //----------------------------------------------------------------------------
  inline opening::opening(const real t, const bool tc, const uint n, const real m)
    : TH_CONST(tc),
      TH0(fasttree::min(one,fasttree::abs(t))), ITH0(one/TH0),
      N(0), Z(0), lM(0)
  {
    if(!TH_CONST) set_table(n,m);
  }
  //----------------------------------------------------------------------------
  inline void opening::reset(const real t, const bool tc,
			     const uint n, const real m)
  {
    TH_CONST = tc;
    TH0      = fasttree::abs(t);
    ITH0     = one/TH0;
    if(!TH_CONST) set_table(n,m);
  }
  //----------------------------------------------------------------------------
  inline real opening::theta(const real M) const
  {
    if(TH_CONST) return TH0;
    if(TH0==one) return one;
    if(M<=zero ) return one;
    if(M>=one  ) return TH0;
    return tabulated_theta(M);
  }
  //----------------------------------------------------------------------------
  inline real opening::invtheta(const real M) const
  {
    if(TH_CONST) return ITH0;
    if(TH0==one) return one;
    if(M<=zero ) return one;
    if(M>=one  ) return ITH0;
    return tabulated_invtheta(M);
  }
}
////////////////////////////////////////////////////////////////////////////////
#endif // included_open_h
