// -*- C++ -*- std header                                                      |
//-----------------------------------------------------------------------------+
//                                                                             |
// pext.h                                                                      |
//                                                                             |
// C++ code                                                                    |
//                                                                             |
// Copyright Walter Dehnen, 2000-2001                                          |
// e-mail:   dehnen@mpia.de                                                    |
// address:  Max-Planck Institut fr Astronomie,                               |
//           Knigstuhl 17, D-69117 Heidelberg, Germany                        |
//                                                                             |
//-----------------------------------------------------------------------------+
//                                                                             |
// defines                                                                     |
//                                                                             |
// class potext              fully abstract                                    |
//                                                                             |
//-----------------------------------------------------------------------------+

#ifndef included_pext_h
#define included_pext_h

#ifndef included_auxx_h
#include <tree/auxx.h>
#endif

////////////////////////////////////////////////////////////////////////////////
namespace fasttree {
  class potext { // abstract base class for external potential.
  protected:
    typedef real tensor[NDIM][NDIM];
  public:
    virtual vect force(const vect&, const real)       const=0;// F   given (x,t)
    virtual real pot_f(vect&, const vect&, const real)const=0;// Phi and F (x,t)
    virtual real rho  (const vect&, const real)       const=0;// rho given (x,t)
    virtual void energies(                        // contribution to totals in:
			  real  &u,               // scalar pot energy
			  tensor&w,               //   pot energy tensor
			  tensor&t,               // 2*kin energy tensor
			  vect  &x,               // dipole
			  vect  &v,               // momentum
			  amom  &l,               // angular momentum
			  real  &m) const  {}     // mass
  };
}
////////////////////////////////////////////////////////////////////////////////

#endif // included_pext_h
