// -*- C++ -*- std header                                                      |
//-----------------------------------------------------------------------------+
//                                                                             |
// tiny.h                                                                      |
//                                                                             |
// C++ code                                                                    |
//                                                                             |
// Copyright Walter Dehnen, 2000-2001                                          |
// e-mail:   dehnen@mpia.de                                                    |
// address:  Max-Planck Institut fr Astronomie,                               |
//           Knigstuhl 17, D-69117 Heidelberg, Germany                        |
//                                                                             |
//-----------------------------------------------------------------------------+
//                                                                             |
// defines                                                                     |
//                                                                             |
//                                                                             |
//-----------------------------------------------------------------------------+

#ifndef included_tiny_h
#define included_tiny_h

#ifndef included_iostream
#include <iostream>
#define included_iostream
#endif

////////////////////////////////////////////////////////////////////////////////
namespace fasttree {
#define TMI template<class INT>
#define TMF template<class INT> friend
  //////////////////////////////////////////////////////////////////////////////
  // class fastree::tiny-index                                                  
  // for a 8-bit unsigned integer without multiplication and division operators 
  // smallest and largest tiny_index = 0, 255                                   
  //////////////////////////////////////////////////////////////////////////////
  class tiny_index {
  private:
#define SELF tiny_index
    unsigned char X;
  public:
    // 1   type conversions                                                     
        operator unsigned int      ()       { return (unsigned int)(X); }
        operator const unsigned int() const { return (unsigned int)(X); }
    TMI operator INT      ()       { return (INT)(X); }
    TMI operator const INT() const { return (INT)(X); }
//     operator bool()                       { return bool(X); }
//     operator const bool() const           { return bool(X); }
//     operator unsigned int()               { return (unsigned int)(X); }
//     operator const unsigned int() const   { return (unsigned int)(X); }
//     operator unsigned short()             { return (unsigned short)(X); }
//     operator const unsigned short() const { return (unsigned short)(X); }
    // 2   constructors                                                         
        SELF  () {}
        SELF  (const SELF&t) : X(t.X) {}
    TMI SELF  (const INT &i) : X(i)   {}
    // 3   binary operators                                                     
    // 3.1 non-const binary operators                                           
        SELF& operator = (const SELF&t)              { X =t.X; return *this; }
        SELF& operator+= (const SELF&t)              { X+=t.X; return *this; }
        SELF& operator-= (const SELF&t)              { X-=t.X; return *this; }
        SELF& operator&= (const SELF&t)              { X&=t.X; return *this; }
        SELF& operator|= (const SELF&t)              { X|=t.X; return *this; }
        SELF& operator^= (const SELF&t)              { X^=t.X; return *this; }
        SELF& operator<<=(const SELF&t)              { X<<=t.X; return *this; }
        SELF& operator>>=(const SELF&t)              { X>>=t.X; return *this; }
    TMI SELF& operator = (const INT &i)              { X =i; return *this; }
    TMI SELF& operator+= (const INT &i)              { X+=i; return *this; }
    TMI SELF& operator-= (const INT &i)              { X-=i; return *this; }
    TMI SELF& operator<<=(const INT &i)              { X<<=i; return *this; }
    TMI SELF& operator>>=(const INT &i)              { X>>=i; return *this; }
    TMI SELF& operator&= (const INT &i)              { X&=i; return *this; }
    TMI SELF& operator|= (const INT &i)              { X|=i; return *this; }
    TMI SELF& operator^= (const INT &i)              { X^=i; return *this; }
    // 3.2 const binary operators                                               
        SELF  operator+  (const SELF&t) const        { return SELF(X+t.X); }
        SELF  operator-  (const SELF&t) const        { return SELF(X-t.X); }
        SELF  operator&  (const SELF&t) const        { return SELF(X&t.X); }
        SELF  operator|  (const SELF&t) const        { return SELF(X|t.X); }
        SELF  operator^  (const SELF&t) const        { return SELF(X^t.X); }
        SELF  operator<< (const SELF&t) const        { return SELF(X<<t.X); }
        SELF  operator>> (const SELF&t) const        { return SELF(X>>t.X); }
        bool  operator== (const SELF&t) const        { return X == t.X; }
        bool  operator!= (const SELF&t) const        { return X != t.X; }
        bool  operator<  (const SELF&t) const        { return X <  t.X; }
        bool  operator<= (const SELF&t) const        { return X <= t.X; }
        bool  operator>  (const SELF&t) const        { return X >  t.X; }
        bool  operator>= (const SELF&t) const        { return X >= t.X; }
    TMI SELF  operator+  (const INT &i) const        { return SELF(X+i); }
    TMI SELF  operator-  (const INT &i) const        { return SELF(X-i); }
    TMI SELF  operator<< (const INT &i) const        { return SELF(X<<i); }
    TMI SELF  operator>> (const INT &i) const        { return SELF(X>>i); }
    TMI SELF  operator&  (const INT &i) const        { return SELF(X&i); }
    TMI SELF  operator|  (const INT &i) const        { return SELF(X|i); }
    TMI SELF  operator^  (const INT &i) const        { return SELF(X^i); }
    TMI bool  operator== (const INT &i) const        { return X == i; }
    TMI bool  operator!= (const INT &i) const        { return X != i; }
    TMI bool  operator<  (const INT &i) const        { return X <  i; }
    TMI bool  operator<= (const INT &i) const        { return X <= i; }
    TMI bool  operator>  (const INT &i) const        { return X >  i; }
    TMI bool  operator>= (const INT &i) const        { return X >= i; }
    // 3.3 friend binary operators                                              
    TMF SELF  operator&  (const INT&i, const SELF&t) {return t&i; }
    TMF SELF  operator|  (const INT&i, const SELF&t) {return t|i; }
    TMF SELF  operator^  (const INT&i, const SELF&t) {return t^i; }
    TMF bool  operator== (const INT&i, const SELF&t) {return i == t.X; }
    TMF bool  operator!= (const INT&i, const SELF&t) {return i != t.X; }
    TMF bool  operator<  (const INT&i, const SELF&t) {return i <  t.X; }
    TMF bool  operator<= (const INT&i, const SELF&t) {return i <= t.X; }
    TMF bool  operator>  (const INT&i, const SELF&t) {return i >  t.X; }
    TMF bool  operator>= (const INT&i, const SELF&t) {return i >= t.X; }
    // 4   unary operators                                                      
        SELF& operator++ ()          { ++X; return *this; }
        SELF& operator-- ()          { --X; return *this; }
        SELF  operator++ (int)       { SELF t(*this); ++X; return t; }
        SELF  operator-- (int)       { SELF t(*this); --X; return t; }
        SELF  operator~  ()    const { return SELF(~X); }
        SELF  operator+  ()    const { return SELF(+X); }
    // 5   I/O                                                                  
    friend ostream& operator<<(ostream&o, const SELF&t)
    {
      return o<<(unsigned int)(t);
    }
    friend istream& operator>>(istream&i, SELF&t)
    {
      register int x;
      i >> x;
      t = SELF(x);
      return i;
    }
#undef SELF
  };
  //////////////////////////////////////////////////////////////////////////////
  // class fastree::tiny                                                        
  // for a 8-bit signed integer                                                 
  // smallest and largest tiny = -128, 127                                      
  //////////////////////////////////////////////////////////////////////////////
  class tiny
  {
  private:
#define SELF tiny
    char X;
  public:
    // 1   type conversions                                                     
    TMI operator INT      ()       { return (INT)(X); }
    TMI operator const INT() const { return (INT)(X); }
    // 2   constructord                                                         
        SELF  () {}
        SELF  (const SELF&t) : X(t.X)                {}
    TMI SELF  (const INT &i) : X(i)                  {}
    // 3   binary operators                                                     
    // 3.1 non-const binary operators                                           
        SELF& operator = (const SELF&t)              { X =t.X; return *this; }
        SELF& operator+= (const SELF&t)              { X+=t.X; return *this; }
        SELF& operator-= (const SELF&t)              { X-=t.X; return *this; }
        SELF& operator*= (const SELF&t)              { X*=t.X; return *this; }
        SELF& operator/= (const SELF&t)              { X/=t.X; return *this; }
        SELF& operator&= (const SELF&t)              { X&=t.X; return *this; }
        SELF& operator|= (const SELF&t)              { X|=t.X; return *this; }
        SELF& operator^= (const SELF&t)              { X^=t.X; return *this; }
        SELF& operator<<=(const SELF&t)              { X<<=t.X; return *this; }
        SELF& operator>>=(const SELF&t)              { X>>=t.X; return *this; }
    TMI SELF& operator = (const INT &i)              { X =i; return *this; }
    TMI SELF& operator+= (const INT &i)              { X+=i; return *this; }
    TMI SELF& operator-= (const INT &i)              { X-=i; return *this; }
    TMI SELF& operator*= (const INT &i)              { X*=i; return *this; }
    TMI SELF& operator/= (const INT &i)              { X/=i; return *this; }
    TMI SELF& operator<<=(const INT &i)              { X<<=i; return *this; }
    TMI SELF& operator>>=(const INT &i)              { X>>=i; return *this; }
    TMI SELF& operator&= (const INT &i)              { X&=i; return *this; }
    TMI SELF& operator|= (const INT &i)              { X|=i; return *this; }
    TMI SELF& operator^= (const INT &i)              { X^=i; return *this; }
    // 3.2 const binary operators                                               
        SELF  operator+  (const SELF&t) const        { return SELF(X+t.X); }
        SELF  operator-  (const SELF&t) const        { return SELF(X-t.X); }
        SELF  operator*  (const SELF&t) const        { return SELF(X*t.X); }
        SELF  operator/  (const SELF&t) const        { return SELF(X/t.X); }
        SELF  operator&  (const SELF&t) const        { return SELF(X&t.X); }
        SELF  operator|  (const SELF&t) const        { return SELF(X|t.X); }
        SELF  operator^  (const SELF&t) const        { return SELF(X^t.X); }
        SELF  operator<< (const SELF&t) const        { return SELF(X<<t.X); }
        SELF  operator>> (const SELF&t) const        { return SELF(X>>t.X); }
        bool  operator== (const SELF&t) const        { return X == t.X; }
        bool  operator!= (const SELF&t) const        { return X != t.X; }
        bool  operator<  (const SELF&t) const        { return X <  t.X; }
        bool  operator<= (const SELF&t) const        { return X <= t.X; }
        bool  operator>  (const SELF&t) const        { return X >  t.X; }
        bool  operator>= (const SELF&t) const        { return X >= t.X; }
    TMI SELF  operator+  (const INT &i) const        { return SELF(X+i); }
    TMI SELF  operator-  (const INT &i) const        { return SELF(X-i); }
    TMI SELF  operator*  (const INT &i) const        { return SELF(X*i); }
    TMI SELF  operator/  (const INT &i) const        { return SELF(X/i); }
    TMI SELF  operator<< (const INT &i) const        { return SELF(X<<i); }
    TMI SELF  operator>> (const INT &i) const        { return SELF(X>>i); }
    TMI SELF  operator&  (const INT &i) const        { return SELF(X&i); }
    TMI SELF  operator|  (const INT &i) const        { return SELF(X|i); }
    TMI SELF  operator^  (const INT &i) const        { return SELF(X^i); }
    TMI bool  operator== (const INT &i) const        { return X == i; }
    TMI bool  operator!= (const INT &i) const        { return X != i; }
    TMI bool  operator<  (const INT &i) const        { return X <  i; }
    TMI bool  operator<= (const INT &i) const        { return X <= i; }
    TMI bool  operator>  (const INT &i) const        { return X >  i; }
    TMI bool  operator>= (const INT &i) const        { return X >= i; }
    // 3.3 friend binary operators                                              
    TMF SELF  operator&  (const INT&i, const SELF&t) {return t&i; }
    TMF SELF  operator|  (const INT&i, const SELF&t) {return t|i; }
    TMF SELF  operator^  (const INT&i, const SELF&t) {return t^i; }
    TMF bool  operator== (const INT&i, const SELF&t) {return i == t.X; }
    TMF bool  operator!= (const INT&i, const SELF&t) {return i != t.X; }
    TMF bool  operator<  (const INT&i, const SELF&t) {return i <  t.X; }
    TMF bool  operator<= (const INT&i, const SELF&t) {return i <= t.X; }
    TMF bool  operator>  (const INT&i, const SELF&t) {return i >  t.X; }
    TMF bool  operator>= (const INT&i, const SELF&t) {return i >= t.X; }
    // 4   unary operators                                                      
        SELF& operator++ ()          { ++X; return *this; }
        SELF& operator-- ()          { --X; return *this; }
        SELF  operator++ (int)       { SELF t(*this); ++X; return t; }
        SELF  operator-- (int)       { SELF t(*this); --X; return t; }
        SELF  operator~  ()    const { return SELF(~X); }
        SELF  operator-  ()    const { return SELF(-X); }
        SELF  operator+  ()    const { return SELF(+X); }
    // 5   I/O                                                                  
    friend ostream& operator<<(ostream&o, const SELF&t)
    {
      return o<<int(t);
    }
    friend istream& operator>>(istream&i, SELF&t)
    {
      register int x;
      i>>x;
      t = SELF(x);
      return i;
    }
#undef SELF
  };
}
#undef TMF
#undef TMI
////////////////////////////////////////////////////////////////////////////////
#endif // included_tiny_h
