
#include <fstream>
#include <iomanip>
#define DOUBLEPRECISION
#include <tree/auxx.h>
#include <tree/gren.h>
extern "C" {
#include <time.h>
#include <math.h>
#include <stdlib.h>
}

using namespace std;
using namespace fasttree;

int main(int argc, char* argv[])
{
  if(argc != 3 && argc != 4) {
    cerr<<
      " \"Direct K EPS [Nx]\"\n"
      " - reads N & N times [m, x,y,z] from stdin\n"
      " - computes by direct summation in double precision\n"
      "   potential and acceleration and writes them to stdout\n"
      " K     = kernel code\n"
      " EPS   = softening length\n"
      " Nx    = compute first Nx gravities, defaulf Nx=N\n";
    exit(1);
  }
  register unsigned i,j;
  register int  KER = atoi(argv[1]);
  register real EPS = atof(argv[2]);
  unsigned  N, Nx;
  // input of masses and positions
  cin >>N;
  real    *M=new real[N];
  vect    *X=new vect[N];
  for(i=0; i!=N;  ++i) cin>>M[i]>>X[i];
  // prepare for computations
  register real
    EPQ = EPS*EPS,
    IEP = (EPS==zero)? one : one/EPS,
    IEQ = IEP*IEP;
  Nx = (argc==4)? min(N,unsigned(atoi(argv[3]))) : N;
  real    *P=new real[Nx];
  vect    *A=new vect[Nx];
  register clock_t cpu0 = clock(), cpu1;
  for(i=0; i!=Nx; ++i) { P[i] = zero; A[i] = zero; }
  register vect R;
  register real Rq,x,D0,ME,MUM,time=zero;
  // do computations
#if 0
#define LOOPDIRECT(GRAVITY)			\
  for(i=0; i!=Nx; ++i) {			\
    for(j=0; j!=i; ++j) {			\
      R    = X[i] - X[j];			\
      Rq   = norm(R);				\
      MUM  = M[i]*M[j];				\
      GRAVITY					\
      P[i]-= D0;				\
      P[j]-= D0;				\
      A[i]+= R;					\
      A[j]-= R;					\
    }						\
    cpu1 = clock();				\
    time+= (cpu1-cpu0) / real(CLOCKS_PER_SEC);	\
    cpu0 = cpu1;				\
  }						\
  for(i=Nx; i!=N; ++i) {			\
    for(j=0; j!=Nx; ++j) {			\
      R    = X[i] - X[j];			\
      Rq   = norm(R);				\
      MUM  = M[i]*M[j];				\
      GRAVITY					\
      P[j]-= D0;				\
      A[j]-= R;					\
    }						\
    cpu1 = clock();				\
    time+= (cpu1-cpu0) / real(CLOCKS_PER_SEC);	\
    cpu0 = cpu1;				\
  }						\
  for(i=0; i!=Nx; ++i) {			\
    ME    = one/M[i];				\
    P[i] *= ME;					\
    A[i] *= ME;					\
  }						\
  cpu1 = clock();				\
  time+= (cpu1-cpu0) / real(CLOCKS_PER_SEC);	\
  cpu0 = cpu1;
#endif
#define LOOPDIRECT(GRAVITY)			\
  for(i=0; i!=Nx; ++i) {			\
    for(j=i+1; j!=Nx; ++j) {			\
      R    = X[i] - X[j];			\
      Rq   = norm(R);				\
      MUM  = M[i]*M[j];				\
      GRAVITY					\
      P[i]-= D0;				\
      P[j]-= D0;				\
      A[i]+= R;					\
      A[j]-= R;					\
    }						\
    for(j=Nx; j!=N; ++j) {			\
      R    = X[i] - X[j];			\
      Rq   = norm(R);				\
      MUM  = M[i]*M[j];				\
      GRAVITY					\
      P[i]-= D0;				\
      A[i]+= R;					\
    }						\
    ME    = one/M[i];				\
    A[i] *= ME;					\
    P[i] *= ME;					\
    cpu1 = clock();				\
    time+= (cpu1-cpu0) / real(CLOCKS_PER_SEC);	\
    cpu0 = cpu1;				\
  }
  switch(KER) {
  case 0:  { LOOPDIRECT(HARDSOFT_GLOB(MUM,F0)) } break;
  case 1:  { LOOPDIRECT(HARDSOFT_GLOB(MUM,F1)) } break;
  case 2:  { LOOPDIRECT(HARDSOFT_GLOB(MUM,F2)) } break;
  case 3:  { LOOPDIRECT(HARDSOFT_GLOB(MUM,F3)) } break;
  case 10: { LOOPDIRECT(HARDSOFT_GLOB(MUM,K0)) } break;
  case 11: { LOOPDIRECT(HARDSOFT_GLOB(MUM,K1)) } break;
  case 12: { LOOPDIRECT(HARDSOFT_GLOB(MUM,K2)) } break;
  case 13: { LOOPDIRECT(HARDSOFT_GLOB(MUM,K3)) } break;
  case 20: { LOOPDIRECT(P0(MUM))               } break;
  case 21: { LOOPDIRECT(P1(MUM))               } break;
  case 22: { LOOPDIRECT(P2(MUM))               } break;
  case 23: { LOOPDIRECT(P3(MUM))               } break;
  default: { LOOPDIRECT(NEWTON(MUM))           } break;
  }
#undef LOOPDIRECT
  // output of potentials and accelerations
  cout<< Nx<<"\n";
  for(i=0; i!=Nx; ++i) cout<<setprecision(10)<<P[i]<<" "<<A[i]<<"\n";
  cout.flush();
  delete[] M;
  delete[] P;
  delete[] X;
  delete[] A;
  cerr<<" Direct finished; time needed for computations: "<<time<<" sec\n";
}
  

