
#include <iostream>
#include <iomanip>
#include <math.h>
#include <body.h>
#include <falcON.h>
// #include <tree/grav.h>
// #include <tree/stat.h>
#include <time.h>
extern "C" {
#include <stdlib.h>
}

using namespace std;
using namespace fasttree;

int main(int argc, char* argv[])
{
  if(argc != 7) {
    cerr<<
      " \"SpecialTest N S THE C Nc Nx\"\n"
      " Test similar to that of Cheng Greengard & Rokhlin (1999)\n"
      " N          # bodies\n"
      " S = long   random seed\n"
      " THE        opening parameter; if < 0, use theta=const\n"
      " C = (0/1)  center root on origin or not\n"
      " Nc         Ncrit\n"
      " Nx         # bodies used to compare with direct summation\n";
    exit(1);
  }

  unsigned N    = atoi(argv[1]);
  long     Seed = atoi(argv[2]);
  real     TH   = atof(argv[3]);
  int      C    = atoi(argv[4]);
  unsigned Nc   = atoi(argv[5]);
  unsigned Nx   = atoi(argv[6]);

  register clock_t      cpu0, cpu1;

  srand48(Seed);
  body  *B = new body[N];
  vect origin=0., *porigin = C? &origin : 0;
  register body *Bi;
  register unsigned i,d;
  register real mi = one/real(N);
  for(Bi=B,i=0; i!=N; ++Bi,++i) {
    for(d=0; d<NDIM; ++d)
      Bi->ppos(d) = drand48()-0.5;
    Bi->mass() = mi;
    Bi->flag_as_sink();
  }

  cpu0 = clock();
  falcON FALCON(B,N,zero,TH,newton);
  FALCON.grow(Nc,porigin);
  FALCON.approximate_gravity();
  cpu1 = clock();

  FALCON.stats(cout);

  cerr<<" time needed by tree:   "<<(cpu1 - cpu0) / real(CLOCKS_PER_SEC)<<" sec\n";

#define dvect vector<double>

  register unsigned j;
  dvect   *X = new dvect[N];
  double  *M = new double[N];
  dvect   *A = new dvect[Nx];
  double  *P = new double[Nx];
  register dvect  R;
  register double x,Pot;
  for(i=0,Bi=B; i!=N; ++i,++Bi) {
    M[i]    = mass(Bi);
    X[i][0] = ppos(Bi)[0];
    X[i][1] = ppos(Bi)[1];
    X[i][2] = ppos(Bi)[2];
  }

  cpu0 = clock();
  for(i=0; i!=Nx; ++i) {
    P[i]    = zero;
    A[i]    = zero;
  }
  for(i=0; i!=Nx; ++i) {
    for(j=i+1; j!=Nx; ++j) {
      R     = X[i]-X[j];
      x     = one/norm(R);
      Pot   =-M[i]*M[j]*sqrt(x);
      R    *= Pot*x;
      P[i] += Pot;
      P[j] += Pot;
      A[i] += R;
      A[j] -= R;
    }
    for(j=Nx; j!=N; ++j) {
      R     = X[i]-X[j];
      x     = one/norm(R);
      Pot   =-M[i]*M[j]*sqrt(x);
      R    *= Pot*x;
      P[i] += Pot;
      A[i] += R;
    }
    x     = one/M[i];
    A[i] *= x;
    P[i] *= x;
  }
  cpu1 = clock();

  register double rmsdP=0,rmsP=0,rmsdA=0,rmsA=0;
  for(i=0,Bi=B; i!=Nx; ++i,++Bi) {
    rmsP += square(P[i]);
    rmsdP+= square(P[i] - pot(Bi));
    rmsA += norm(A[i]);
    rmsdA+= square(A[i][0] - acc(Bi)[0])
           +square(A[i][1] - acc(Bi)[1])
           +square(A[i][2] - acc(Bi)[2]);
  }
  register real time_dir=(cpu1 - cpu0) / real(CLOCKS_PER_SEC);
  cerr<<" time needed by direct: "<<time_dir<<" sec\n";
  time_dir *= 0.5*N*(N-1)/(0.5*Nx*(Nx-1)+Nx*(N-Nx));
  cerr<<" time for all direct:   "<<time_dir<<" sec\n\n"
      <<" E_pot = "<<sqrt(rmsdP/rmsP)<<"\n"
      <<" E_acc = "<<sqrt(rmsdA/rmsA)<<"\n";
}
